@@ -4,7 +4,7 @@
  * ----------------------------------------------------------------------------
  * Mastering programed by YAMASHINA Hio
  * ----------------------------------------------------------------------------
- * $Id: conv.c,v 1.9 2005/08/15 08:58:42 hio Exp $
+ * $Id: conv.c,v 1.11 2006/07/03 01:33:15 hio Exp $
  * ------------------------------------------------------------------------- */
 
 #ifdef _MSC_VER
@@ -49,18 +49,18 @@ SV*
 xs_sjis_utf8(SV* sv_str)
 {
   STRLEN src_len;
-  unsigned char* src;
+  UJ_UINT8* src;
   int len;
   
   SV_Buf result;
-  const unsigned char* src_end;
+  const UJ_UINT8* src_end;
   
   if( sv_str==&PL_sv_undef )
   {
     return newSVsv(&PL_sv_undef);
   }
   
-  src = (unsigned char*)SvPV(sv_str,src_len);
+  src = (UJ_UINT8*)SvPV(sv_str,src_len);
   len = sv_len(sv_str);
 #if DISP_S2U
   fprintf(stderr,"Unicode::Japanese::(xs)sjis_utf8\n",len);
@@ -71,7 +71,7 @@ xs_sjis_utf8(SV* sv_str)
 
   while( src<src_end )
   {
-    const unsigned char* ptr;
+    const UJ_UINT8* ptr;
     if( src[0]<0x80 )
     { /* ASCII */
       ECHO_U2S((stderr,"ascii: %02x\n",src[0]));
@@ -81,19 +81,19 @@ xs_sjis_utf8(SV* sv_str)
     }else if( 0xa1<=src[0] && src[0]<=0xdf )
     { /* half-width katakana (ja:Ⱦѥ) */
       ECHO_U2S((stderr,"kana: %02x\n",src[0]));
-      ptr = (unsigned char*)&g_s2u_table[(src[0]-0xa1)*3];
+      ptr = (UJ_UINT8*)&g_s2u_table[(src[0]-0xa1)*3];
       ++src;
     }else if( src+1<src_end && 0x81<=src[0] && src[0]<=0x9f )
     { /* a double-byte letter (ja:2Хʸ) */
-      const unsigned short sjis = ntohs(*(unsigned short*)src);
+      const UJ_UINT16 sjis = (src[0]<<8)+src[1]; /* ntohs */
       ECHO_U2S((stderr,"sjis.dbcs#1: %04x\n",sjis));
-      ptr = (unsigned char*)&g_s2u_table[(sjis - 0x8100 + 0x3f)*3];
+      ptr = (UJ_UINT8*)&g_s2u_table[(sjis - 0x8100 + 0x3f)*3];
       src += 2;
     }else if( src+1<src_end && 0xe0<=src[0] && src[0]<=0xfc )
     { /* a double-byte letter (ja:2Хʸ) */
-      const unsigned short sjis = ntohs(*(unsigned short*)src);
+      const UJ_UINT16 sjis = (src[0]<<8)+src[1]; /* ntohs */
       ECHO_U2S((stderr,"sjis.dbcs#2: %04x\n",sjis));
-      ptr = (unsigned char*)&g_s2u_table[(sjis- 0xe000 + 0x1f3f)*3];
+      ptr = (UJ_UINT8*)&g_s2u_table[(sjis- 0xe000 + 0x1f3f)*3];
       src += 2;
     }else
     { /* unknown */
@@ -108,11 +108,11 @@ xs_sjis_utf8(SV* sv_str)
     if( ptr[2] )
     {
       /*fprintf(stderr,"utf8-len: [%d]\n",3); */
-      SV_Buf_append_ch3(&result,*(int*)ptr);
+      SV_Buf_append_mem(&result, ptr, 3);
     }else if( ptr[1] )
     {
       /*fprintf(stderr,"utf8-len: [%d]\n",2); */
-      SV_Buf_append_ch2(&result,*(short*)ptr);
+      SV_Buf_append_mem(&result, ptr, 2);
     }else if( ptr[0] )
     {
       /*fprintf(stderr,"utf8-len: [%d]\n",1); */
@@ -138,16 +138,18 @@ EXTERN_C
 SV*
 xs_utf8_sjis(SV* sv_str)
 {
-  unsigned char* src;
+  const UJ_UINT8* src;
   int len;
   SV_Buf result;
-  const unsigned char* src_end;
+  const UJ_UINT8* src_end;
+  static const UJ_UINT8 char_null[2]    = { '\0', '\0' };
+  static const UJ_UINT8 char_unknown[2] = { '?',  '\0' };
   
   if( sv_str==&PL_sv_undef )
   {
     return newSVsv(&PL_sv_undef);
   }
-  src = (unsigned char*)SvPV(sv_str,PL_na);
+  src = (UJ_UINT8*)SvPV(sv_str,PL_na);
   len = sv_len(sv_str);
 
   ECHO_U2S((stderr,"Unicode::Japanese::(xs)utf8_sjis\n"));
@@ -169,7 +171,7 @@ xs_utf8_sjis(SV* sv_str)
       {
         ++len;
       }
-      SV_Buf_append_str(&result,src,len);
+      SV_Buf_append_mem(&result,src,len);
       src+=len;
       continue;
     }
@@ -177,9 +179,9 @@ xs_utf8_sjis(SV* sv_str)
     /* non-ascii */
     if( 0xe0<=*src && *src<=0xef )
     { /* 3byte range. mostly enter here. */
-      const int          utf8_len = 3;
-      const unsigned int ucs_min  = 0x800;
-      const unsigned int ucs_max  = 0xffff;
+      const int       utf8_len = 3;
+      const UJ_UINT32 ucs_min  = 0x800;
+      const UJ_UINT32 ucs_max  = 0xffff;
       ECHO_U2S((stderr,"utf8-len: [%d]\n",utf8_len));
       /* check length */
       if( src+utf8_len<=src_end )
@@ -213,9 +215,9 @@ xs_utf8_sjis(SV* sv_str)
       /* ok. */
     }else if( 0xf0<=*src && *src<=0xf7 )
     {
-      const int          utf8_len = 4;
-      const unsigned int ucs_min  = 0x010000;
-      const unsigned int ucs_max  = 0x10ffff;
+      const int       utf8_len = 4;
+      const UJ_UINT32 ucs_min  = 0x010000;
+      const UJ_UINT32 ucs_max  = 0x10ffff;
       ECHO_U2S((stderr,"utf8-len: [%d]\n",utf8_len));
       /* check length */
       if( src+utf8_len<=src_end )
@@ -263,9 +265,9 @@ xs_utf8_sjis(SV* sv_str)
       }
     }else if( 0xc0<=*src && *src<=0xdf )
     {
-      const int          utf8_len = 2;
-      const unsigned int ucs_min  =  0x80;
-      const unsigned int ucs_max  = 0x7ff;
+      const int       utf8_len = 2;
+      const UJ_UINT32 ucs_min  =  0x80;
+      const UJ_UINT32 ucs_max  = 0x7ff;
       ECHO_U2S((stderr,"utf8-len: [%d]\n",utf8_len));
       /* check length */
       if( src+utf8_len<=src_end )
@@ -373,24 +375,24 @@ xs_utf8_sjis(SV* sv_str)
     {
       sjis_ptr = g_u2s_table + (ucs - 0xF900 + 0xA000)*2;
     }else if( 0x0FE000<=ucs && ucs<=0x0FFFFF )
-    {
-      sjis_ptr = "?"; /* exactly 2byte: "?\0" */
+    { /* emoji. */
+      sjis_ptr = char_unknown; /* "?\0" */
     }else
     {
-      sjis_ptr = "\0"; /* exactly 2byte: "\0\0" */
+      sjis_ptr = char_null; /* "\0\0" */
     }
-    if( *(const UJ_UINT16*)sjis_ptr != 0 )
+    if( sjis_ptr[0]!=0 || sjis_ptr[1]!=0 )
     { /* mapping dest exists. */
       if( sjis_ptr[1]!=0 )
       {
-        SV_Buf_append_ch2(&result,*(UJ_UINT16*)sjis_ptr);
+        SV_Buf_append_mem(&result, sjis_ptr, 2);
       }else
       {
-        SV_Buf_append_ch(&result,sjis_ptr[0]);
+        SV_Buf_append_ch(&result, *sjis_ptr);
       }
     }else if( ucs<=0x7F )
     {
-      SV_Buf_append_ch(&result,(unsigned char)ucs);
+      SV_Buf_append_ch(&result,(UJ_UINT8)ucs);
     }else
     {
       SV_Buf_append_entityref(&result,ucs);
